package org.vraptor;

import java.io.IOException;

import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.log4j.Logger;
import org.vraptor.component.ComponentNotFoundException;
import org.vraptor.component.LogicNotFoundException;
import org.vraptor.config.ConfigException;
import org.vraptor.core.ControllerFactory;
import org.vraptor.core.VRaptorController;
import org.vraptor.interceptor.InterceptorInstantiationException;
import org.vraptor.url.InvalidURLException;
import org.vraptor.view.ViewException;

/**
 * VRaptor servlet contains a vraptor controller capable of answering web
 * requests.
 * 
 * @author Guilherme Silveira
 */
public class VRaptorServlet extends HttpServlet {

	private static final Logger LOG = Logger.getLogger(VRaptorServlet.class);

	/** serialversion uid */
	private static final long serialVersionUID = 1084551538406918486L;

	private VRaptorController controller;

	@Override
	public void init() throws ServletException {
		super.init();
		try {
			this.controller = new ControllerFactory()
					.configure(getServletContext());
		} catch (ConfigException e) {
			throw new ServletException("Unable to configure vraptor2", e);
		}
	}

	@Override
	protected void service(HttpServletRequest request,
			HttpServletResponse response) throws ServletException, IOException {

		LOG.debug("new request to " + request.getRequestURI());
		try {
			controller.execute(request, response);
		} catch (LogicNotFoundException exception) {
			LOG.error("Logic not found", exception);
			response.sendError(404, "URL not found");
		} catch (InvalidURLException ex) {
			throw new ServletException("Unable to deal with selected url", ex);
		} catch (ComponentNotFoundException e) {
			LOG.error("Component not found", e);
			response.sendError(404, "URL not found");
		} catch (InterceptorInstantiationException e) {
			LOG.error("Unable to instantiate interceptors", e);
			throw new ServletException(e.getMessage(), e);
		} catch (ViewException ex) {
			LOG.error("View exception", ex);
			if (ex.getCause() == null) {
				throw new ServletException(ex.getMessage(), ex);
			} else {
				throw new ServletException(ex.getCause().getMessage(), ex
						.getCause());
			}
		} catch (LogicException ex) {
			LOG.error("Logic exception", ex);
			if (ex.getCause() == null) {
				throw new ServletException(ex.getMessage(), ex);
			} else {
				throw new ServletException(ex.getCause().getMessage(), ex
						.getCause());
			}
		}

	}
}
