package org.vraptor.component;

import java.lang.reflect.Constructor;

import org.vraptor.LogicRequest;

/**
 * A component constructor.
 * 
 * @author Guilherme Silveira
 */
class ComponentConstructor {

	private Constructor constructor;

	private Class[] types;

	private String[] keys;

	ComponentConstructor(Constructor constructor) {
		this.constructor = constructor;
		this.types = constructor.getParameterTypes();
		this.keys = new String[types.length];
		for (int i = 0; i < types.length; i++) {
			keys[i] = types[i].getName();
		}
	}

	public Object newInstance(LogicRequest request)
			throws ComponentInstantiationException {
		Object[] values = new Object[keys.length];
		for (int i = 0; i < keys.length; i++) {
			values[i] = request.findAttribute(keys[i]);
			if (values[i] == null) {
				throw new ComponentInstantiationException(
						"Unable to fill value for key " + keys[i]);
			}
		}
		try {
			return constructor.newInstance(values);
		} catch (Exception e) {
			throw new ComponentInstantiationException(
					"Unable to instantiate using " + constructor, e);
		}
	}

	@Override
	public String toString() {
		return constructor.getName() + keys.toString();
	}

}
