package org.vraptor.converter.basic;

import org.vraptor.LogicRequest;
import org.vraptor.converter.ConversionException;
import org.vraptor.converter.Converter;

/**
 * Simple byte converter.
 * Uses the error key invalid_byte if unable to parse its information.
 * 
 * @author Guilherme Silveira
 */
public class SimpleByteConverter implements Converter {

	public Object convert(String value, Class<?> type, LogicRequest context)
			throws ConversionException {
		if (value == null || value.equals("")) {
			return null;
		}
		try {
			return Byte.valueOf(value);
		} catch (NumberFormatException e) {
			throw new ConversionException("invalid_byte", e.getMessage(), e);
		}
	}

	/**
	 * Returns the list of supported types
	 * 
	 * @see org.vraptor.converter.Converter#getSupportedTypes()
	 */
	public Class<?>[] getSupportedTypes() {
		return new Class[] { Byte.class };
	}

}
