package org.vraptor.core;

import javax.servlet.ServletContext;

import org.apache.log4j.Logger;
import org.vraptor.config.ConfigException;
import org.vraptor.webapp.DefaultWebApplication;
import org.vraptor.webapp.WebApplication;

/**
 * VRaptor's controller factory.
 * 
 * @author Guilherme Silveira
 * 
 */
public class ControllerFactory {

	private static final Logger LOG = Logger.getLogger(ControllerFactory.class);

	/**
	 * VRaptor's controller factory.
	 * 
	 * @param servletContext
	 *            the servlet context
	 * @return the new vraptor controller
	 * @throws ConfigException
	 *             configuration exception
	 */
	public VRaptorController configure(ServletContext servletContext) throws ConfigException {
		long start = System.currentTimeMillis();
		WebApplication webApplication = new DefaultWebApplication(servletContext);
		webApplication.init();
		VRaptorController controller = new VRaptorController(webApplication);
		servletContext.setAttribute("webApplication", webApplication);
		webApplication.getPluginManager().init(webApplication);
		long buildTime = System.currentTimeMillis() - start;
		LOG.debug("Vraptor init time " + (buildTime / 1000.0) + " milliseconds");
		return controller;
	}

}
