package org.vraptor.core;

import org.apache.log4j.Logger;
import org.vraptor.Interceptor;
import org.vraptor.LogicException;
import org.vraptor.component.ComponentNotFoundException;
import org.vraptor.component.ComponentType;
import org.vraptor.component.LogicMethod;
import org.vraptor.component.LogicNotFoundException;
import org.vraptor.interceptor.ComponentLookupInterceptor;
import org.vraptor.interceptor.ExecuteLogicInterceptor;
import org.vraptor.interceptor.InjectionInterceptor;
import org.vraptor.interceptor.InterceptorInstantiationException;
import org.vraptor.interceptor.OutjectionInterceptor;
import org.vraptor.interceptor.RegisterAttributesInteceptor;
import org.vraptor.interceptor.SettingAndValidationInterceptor;
import org.vraptor.interceptor.ViewInterceptor;
import org.vraptor.url.SimpleLogicRequest;
import org.vraptor.view.ViewException;

/**
 * VRaptor's engine.
 * 
 * @author Guilherme Silveira
 */
public class VRaptorExecution {

	private static final Logger LOG = Logger.getLogger(VRaptorExecution.class);

	private VRaptorController controller;

	private SimpleLogicRequest request;

	private String result;

	public VRaptorExecution(SimpleLogicRequest logicRequest,
			VRaptorController controller) {
		this.controller = controller;
		this.request = logicRequest;
	}

	public String execute() throws ComponentNotFoundException,
			LogicNotFoundException, ViewException,
			InterceptorInstantiationException, LogicException {

		LOG.debug("Calling execute on " + request.toString());

		ComponentType componentType = this.controller.getWebApplication()
				.getComponentManager().getComponent(request);

		// gets the logic method
		LogicMethod method = componentType.getLogic(request.getLogicName());

		// gets the flow
		// TODO those interceptors should not use member variables to be faster:
		// cache them
		
		Interceptor register = new RegisterAttributesInteceptor(this.request,method);
		
		Interceptor first = new ComponentLookupInterceptor(componentType);
		Interceptor second = new InjectionInterceptor(this.request
				.getLogicContext(), controller.getIntrospector(), componentType);
		Interceptor third = new SettingAndValidationInterceptor(componentType,
				controller.getIntrospector(), this, method);
		Interceptor fourth = new ExecuteLogicInterceptor(this, componentType,
				method);
		Interceptor outjection = new OutjectionInterceptor(this, componentType);
		Interceptor redirect = new ViewInterceptor(controller
				.getWebApplication().getViewManager(), method);

		BasicLogicFlow logicFlow = new BasicLogicFlow(componentType, this,
				new Interceptor[] { register }, new Interceptor[] { first,
						second, third, fourth, outjection, redirect });

		// executes it
		logicFlow.execute();

		// returns its result, if any
		return this.result;

	}

	/**
	 * Returns its request object
	 * 
	 * @return the request
	 */
	public SimpleLogicRequest getRequest() {
		return request;
	}

	/**
	 * Returns vraptor's controller
	 * 
	 * @return the controller
	 */
	public VRaptorController getController() {
		return controller;
	}

	public void confirmResult(String result) {
		this.result = result;
	}

	public void forward(String result) throws ViewException {
		controller.getWebApplication().getViewManager().forward(request,
				result, request.getRequest(), request.getResponse());
		confirmResult(result);
	}

}
