package org.vraptor.core;

import java.util.List;
import java.util.Locale;
import java.util.MissingResourceException;
import java.util.ResourceBundle;

import org.apache.log4j.Logger;
import org.vraptor.LogicRequest;
import org.vraptor.annotations.In;
import org.vraptor.component.FieldAnnotation;
import org.vraptor.component.LogicMethod;
import org.vraptor.introspector.Introspector;
import org.vraptor.reflection.SettingException;
import org.vraptor.validator.UnstableValidationException;
import org.vraptor.validator.ValidationErrors;

/**
 * Responsible for validating data.
 * 
 * @author Guilherme Silveira
 */
public class ValidatorEngine {

	private static final Logger LOG = Logger.getLogger(ValidatorEngine.class);

	private static final String BUNDLE_NAME = "messages";

	public ValidationErrors validate(LogicMethod method, LogicRequest request,
			Introspector introspector, Object component)
			throws SettingException, UnstableValidationException {

		Locale locale = request.getRequestContext().lookupLocale();
		ValidationErrors errors = method.validate(component, request,
				loadBundle(locale));
		return errors;

	}

	private ResourceBundle loadBundle(Locale locale) {
		LOG.debug("loading resource bundle: " + BUNDLE_NAME + " for locale "
				+ locale);
		try {
			return ResourceBundle.getBundle(BUNDLE_NAME, locale);
		} catch (MissingResourceException ex) {
			LOG.debug("Resource bundle not found: " + BUNDLE_NAME);
			return null;
		}
	}
}
