package org.vraptor.interceptor;

import org.vraptor.Interceptor;
import org.vraptor.LogicException;
import org.vraptor.LogicFlow;
import org.vraptor.LogicRequest;
import org.vraptor.component.ComponentInstantiationException;
import org.vraptor.component.ComponentType;
import org.vraptor.scope.Context;
import org.vraptor.scope.LogicContext;
import org.vraptor.scope.ScopeType;
import org.vraptor.view.ViewException;

/**
 * Looks up for the component and registers it in the logic context.
 * 
 * @author Guilherme Silveira
 */
public class ComponentLookupInterceptor implements Interceptor {

	public static final String CURRENT_COMPONENT = "org.vraptor.core.current_component";

	private final ComponentType type;

	public ComponentLookupInterceptor(ComponentType type) {
		this.type = type;
	}

	public void intercept(LogicFlow flow) throws LogicException, ViewException {
		try {
			LogicRequest request = flow.getLogicRequest();
			LogicContext context = request.getLogicContext();

			String key = type.getKey();

			Context scope = type.getScope().getContext(request);

			Object component;
			if (scope.hasAttribute(key)) {
				component = scope.getAttribute(key);
			} else {
				component = type.newInstance(request);
			}

			scope.setAttribute(key, component);
			context.setAttribute(CURRENT_COMPONENT, component);

			flow.execute();

			// if either logic, request or should be removed
			if (type.getDestroyLogicName().equals(request.getLogicName()) || type.getScope().equals(ScopeType.LOGIC)
					|| type.getScope().equals(ScopeType.REQUEST)) {
				scope.removeAttribute(key);
			}

		} catch (ComponentInstantiationException e) {
			throw new LogicException(e.getMessage(), e);
		}
	}
}
