package org.vraptor.interceptor;

import org.apache.log4j.Logger;
import org.vraptor.Interceptor;
import org.vraptor.LogicException;
import org.vraptor.LogicFlow;
import org.vraptor.component.ComponentType;
import org.vraptor.component.LogicMethod;
import org.vraptor.core.VRaptorExecution;
import org.vraptor.introspector.Introspector;
import org.vraptor.reflection.GettingException;
import org.vraptor.view.ViewException;

/**
 * The last interceptor: calls the logic method.
 * 
 * @author Guilherme Silveira
 */
public class ExecuteLogicInterceptor implements Interceptor {

	private static final Logger LOG = Logger
			.getLogger(ExecuteLogicInterceptor.class);

	static final String RESULT_KEY = "org.vraptor.logic.result";

	private VRaptorExecution engine;

	private ComponentType type;

	private LogicMethod logicMethod;

	public ExecuteLogicInterceptor(VRaptorExecution engine,
			ComponentType componentClass, LogicMethod logicMethod) {
		this.engine = engine;
		this.type = componentClass;
		this.logicMethod = logicMethod;
	}

	public void intercept(LogicFlow flow) throws LogicException, ViewException {

		Introspector introspector = engine.getController().getIntrospector();

		LOG.debug("ready to execute logic");

		Object component = engine.getRequest().getLogicContext().getAttribute(
				ComponentLookupInterceptor.CURRENT_COMPONENT);

		// execute
		String result = logicMethod.execute(component, engine.getRequest());
		engine.getRequest().getRequestContext()
				.setAttribute(RESULT_KEY, result);
		try {
			introspector.outject(type.getOutAnnotations(), component, engine
					.getRequest());
		} catch (GettingException e) {
			throw new LogicException(e.getMessage(), e);
		}

		// everything went fine...
		this.engine.confirmResult(result);

		flow.execute();

	}

}
