package org.vraptor.interceptor;

import java.util.List;

import org.apache.log4j.Logger;
import org.vraptor.Interceptor;
import org.vraptor.LogicException;
import org.vraptor.LogicFlow;
import org.vraptor.component.ComponentType;
import org.vraptor.component.LogicMethod;
import org.vraptor.core.ConversionEngine;
import org.vraptor.core.VRaptorExecution;
import org.vraptor.core.ValidatorEngine;
import org.vraptor.i18n.ValidationMessage;
import org.vraptor.introspector.Introspector;
import org.vraptor.reflection.GettingException;
import org.vraptor.reflection.SettingException;
import org.vraptor.validator.ValidationErrors;
import org.vraptor.view.ViewException;

/**
 * Responsible for setters and validation.
 * 
 * @author Guilherme Silveira
 */
public class SettingAndValidationInterceptor implements Interceptor {

	private static final Logger log = Logger.getLogger(SettingAndValidationInterceptor.class);

	private static final String INVALID = "invalid";

	private final ValidatorEngine validatorEngine;

	private final ConversionEngine conversionEngine;

	private final Introspector introspector;

	private final ComponentType type;

	private final VRaptorExecution engine;

	private final LogicMethod method;

	public SettingAndValidationInterceptor(ComponentType type, Introspector introspector, VRaptorExecution engine,
			LogicMethod method) {
		this.validatorEngine = new ValidatorEngine();
		this.conversionEngine = new ConversionEngine();
		this.introspector = introspector;
		this.type = type;
		this.engine = engine;
		this.method = method;
	}

	public void intercept(LogicFlow flow) throws LogicException, ViewException {

		Object component = engine.getRequest().getLogicContext().getAttribute(
				ComponentLookupInterceptor.CURRENT_COMPONENT);

		try {

			List<ValidationMessage> problems = conversionEngine.convert(type, engine, introspector, component);

			ValidationErrors errors = validatorEngine.validate(method, engine.getRequest(), introspector, component);

			for (ValidationMessage problem : problems) {
				errors.add(problem);
			}

			// TODO what the hell? make it a better design..
			if (errors.size() != 0) {
				engine.getRequest().getRequestContext().setAttribute("errors", errors);
				log.debug("Some problems were found");
				// better design... please
				// outjects no matter its ok or not, so it can refill the form
				outjectLogic(introspector, component);
				this.engine.forward(INVALID);
				return;
			}

		} catch (SettingException e) {
			throw new LogicException(e.getMessage(), e);
		}

		flow.execute();
	}

	private void outjectLogic(Introspector introspector, Object component) throws LogicException {
		try {
			introspector.outject(type.getOutAnnotations(), component, engine.getRequest());
		} catch (GettingException e) {
			throw new LogicException(e.getMessage(), e);
		}
	}

}
