package org.vraptor.plugin.hibernate;

import java.util.Set;

import org.apache.log4j.Logger;
import org.vraptor.component.ComponentManager;
import org.vraptor.component.ComponentType;
import org.vraptor.plugin.VRaptorPlugin;
import org.vraptor.webapp.WebApplication;

/**
 * Overrides all components implementing hibernate validation before executing a
 * business logic.
 * 
 * @author Guilherme Silveira
 */
public class HibernateValidatorPlugin implements VRaptorPlugin {

	private static final Logger logger = Logger
			.getLogger(HibernateValidatorPlugin.class);

	private final ValidatorLocator locator;

	public HibernateValidatorPlugin() {
		locator = new ValidatorLocator();
	}

	public void init(WebApplication application) {
		logger.debug("Initializing HibernateValidatorPlugin");
		ComponentManager manager = application.getComponentManager();
		Set<ComponentType> components = manager.getComponents();
		for (ComponentType component : components) {
			manager.register(wrapComponent(component));
		}
	}

	private ComponentType wrapComponent(ComponentType component) {
		return new HibernateComponent(component, locator);
	}

}
