package org.vraptor.scope;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.log4j.Logger;
import org.vraptor.url.SimpleLogicRequest;

/**
 * Basic vraptor logic request.
 * 
 * @author Guilherme Silveira
 */
public class DefaultLogicRequest implements SimpleLogicRequest {

	private static final Logger logger = Logger
			.getLogger(DefaultLogicRequest.class);

	private String logicName;

	private String componentName;

	private HttpServletRequest request;

	private ApplicationContext applicationContext;

	private SessionContext sessionContext;

	private RequestContext requestContext;

	private LogicContext logicContext;

	private HttpServletResponse response;

	/**
	 * Creates the logic context
	 * 
	 * @param request
	 *            request
	 * @param response
	 *            response
	 * @param application
	 *            application scope
	 */
	public DefaultLogicRequest(String componentName, String logicName,
			HttpServletRequest request, HttpServletResponse response,
			ApplicationContext application) {
		logger.debug("logic request created: " + componentName + " "
				+ logicName);
		this.componentName = componentName;
		this.applicationContext = application;
		this.logicName = logicName;
		this.response = response;
		this.request = request;
		this.logicContext = new DefaultLogicContext(request, response);
	}

	public ApplicationContext getApplicationContext() {
		return this.applicationContext;
	}

	public SessionContext getSessionContext() {
		if (this.sessionContext == null) {
			this.sessionContext = new SessionContext(this.request.getSession());
		}
		return this.sessionContext;
	}

	public RequestContext getRequestContext() {
		if (this.requestContext == null) {
			this.requestContext = new DefaultRequestContext(this.request);
		}
		return this.requestContext;
	}

	/**
	 * @see org.vraptor.LogicRequest#getComponentName()
	 */
	public String getComponentName() {
		return this.componentName;
	}

	/**
	 * @see org.vraptor.LogicRequest#getLogicName()
	 */
	public String getLogicName() {
		return this.logicName;
	}

	public HttpServletRequest getRequest() {
		return this.request;
	}

	public HttpServletResponse getResponse() {
		return this.response;
	}

	public LogicContext getLogicContext() {
		return this.logicContext;
	}

	public Object findAttribute(String key) {
		if (getLogicContext().hasAttribute(key)) {
			return logicContext.getAttribute(key);
		}
		if (getRequestContext().hasAttribute(key)) {
			return requestContext.getAttribute(key);
		}
		if (getSessionContext().hasAttribute(key)) {
			return sessionContext.getAttribute(key);
		}
		if (getApplicationContext().hasAttribute(key)) {
			return applicationContext.getAttribute(key);
		}
		return null;
	}

}
