package org.vraptor.scope;

import java.util.Locale;
import java.util.Map;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.jsp.jstl.core.Config;

/**
 * A simple request context.
 * 
 * @author Guilherme Silveira
 */
public class DefaultRequestContext implements Context, RequestContext {

	private final HttpServletRequest request;

	private Locale matchingLocale;

	public DefaultRequestContext(HttpServletRequest request) {
		this.request = request;
	}

	public boolean hasAttribute(String name) {
		return request.getAttribute(name) != null;
	}

	public void setAttribute(String name, Object value) {
		request.setAttribute(name, value);
	}

	public Object getAttribute(String name) {
		return request.getAttribute(name);
	}

	public Object removeAttribute(String name) {
		Object value = request.getAttribute(name);
		request.removeAttribute(name);
		return value;
	}

	@SuppressWarnings("unchecked")
	public Map<String, Object> getParameterMap() {
		return this.request.getParameterMap();
	}

	public Locale lookupLocale() {
		if (this.matchingLocale == null) {
			Locale locale = (Locale) Config.get(request.getSession(),
					Config.FMT_LOCALE);
			if (locale == null) {
				locale = request.getLocale();
			}
			this.matchingLocale = locale;
		}
		return this.matchingLocale;
	}

}
