package org.vraptor.util;

import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.List;

import org.vraptor.annotations.In;
import org.vraptor.annotations.Out;
import org.vraptor.component.ComponentType;
import org.vraptor.plugin.VRaptorPlugin;
import org.vraptor.scope.ScopeType;
import org.vraptor.webapp.WebApplication;

/**
 * Outjects components and plugins variables.
 * 
 * @author Guilherme Silveira
 */
public class StatsQuery {

	@In(scope = ScopeType.APPLICATION)
	private WebApplication webApplication;

	@Out
	private List<ComponentType> components;

	@Out
	private List<VRaptorPlugin> plugins;

	public void read() {
		
		this.components = new ArrayList<ComponentType>(this.webApplication.getComponentManager().getComponents());
		Collections.sort(this.components, new ComponentTypeNameComparator());
		
		this.plugins = this.webApplication.getPluginManager().getPlugins();
	}
	
	/** 
	 * Compares the name of two ComponentType, ignoring case. Note: this comparator
     * imposes orderings that are inconsistent with equals
	 */
	class ComponentTypeNameComparator implements Comparator<ComponentType> {

		public int compare(ComponentType comp1, ComponentType comp2) {
			return comp1.getName().compareToIgnoreCase(comp2.getName());
		}
	}
}
